<?php

declare(strict_types=1);

namespace mirzaev\vk\chat;

// Встроенные библиотеки
use Closure;

/**
 * Команда
 *
 * @package mirzaev\vk\chat
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class command
{
    /**
     * Конструктор
     *
     * @param callable $program Программа выполняемая командой
     * @param array $accounts Реестр аккаунтов которым разрешено выполнять команду
     */
    public function __construct(
        public Closure $program,
        public array $accounts = []
    ) {
    }

    /**
     * Проверить наличие доступа к команде
     *
     * @param int $id Идентификатор аккаунта
     *
     * @return bool Имеет ли аккаунт доступ к команде?
     */
    public function access(int $id): bool
    {
        // Перебор реестра аккаунтов
        foreach ($this->accounts as $account) if ($account === $id) return true;

        return false;
    }

    /**
     * Выполнить команду
     *
     * @param string ...$parameters Параметры команды
     *
     * @return bool
     */
    public function execute(string ...$parameters): bool
    {
        return ($this->program)(...$parameters);
    }
}
