<?php

declare(strict_types=1);

namespace mirzaev\vk\chat;

// Файлы проекта
use mirzaev\vk\chat\command;

/**
 * Шаблон
 *
 * @package mirzaev\vk\chat
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class pattern
{
    /**
     * @var array $commands Команды
     */
    protected array $commands = [];

    /**
     * Конструктор
     *
     * @param string $text Текст
     */
    public function __construct(public string $text)
    {
    }

    /**
     * Записать команду в реестр
     *
     * @param command $command Инстанция команды
     *
     * @return static
     */
    public function command(command $command): static
    {
        // Запись в реестр
        $this->commands .= [$command];

        return $this;
    }

    /**
     * Проверить текст на совпадение с шаблоном
     *
     * @param string $text Текст
     * @param array &$parameters Буфер параметров для команды
     *
     * @return bool Текст совпадает с шаблоном?
     */
    public function check(string $text, array &$parameters = []): bool
    {
        // Проверка на совпадение с шаблоном
        preg_match_all($this->text, $text, $matches);

        // Простой шаблон
        if (count($matches) === 1) return !empty($matches[0]);

        // Удаление первого значения (текст для проверки)
        unset($matches[0]);

        // Сложный шаблон
        foreach ($matches as $match) if (!empty($match[0])) $parameters []= $match[0]; else return false;

        return true;
    }

    /**
     * Обработать подключенные команды
     *
     * @param int $id Идентификатор аккаунта вызывающего выполнение
     * @param string ...$parameters Параметры команды
     *
     * @return void
     */
    public function handle(int $id, string ...$parameters): void
    {
        foreach ($this->commands as $command) {
            // Перебор команд

            // Выполнение команды
            if (!empty($command->accounts) && $command->access($id)) $command->execute(...$parameters);
        }
    }
}
